fourier_version::"0.03", fourier_date::"20.08.1999";

/* Numerical computation of Fourier coefficients of 2*pi periodic functions
   (the functions must be defined on [-pi,pi]).
   No FFT (Fast Fourier Transform)! 
   Currently, only real fourier coefficients are computed. However,
   Fourier coefficients of complex functions on [-pi,pi] can be
   computed, if the underlying scheme system supports complex arithmetics */

/* Normally, use fouriercoef and fouriersyn. The other procedures are
   meant to be internal.*/

fouriersin(f,n,nsub)::
  gaussquad(8,x->f(x)*sin(n*x),-pi,pi,nsub);

fouriercos(f,n,nsub)::
  gaussquad(8,x->f(x)*cos(n*x),-pi,pi,nsub);

fourier0(f,nsub)::gaussquad(8,f,-pi,pi,nsub)/2;

fouriercoef(f,n,nsub)::            *** computes constant, sin and cos coeffi-
with c::[0..n,1..2] do             *** cients up to order n using numerical
  c[0,1] := fourier0(f,nsub),      *** integration (order 8 Gauss rule) on nsub
  c[0,2] := 0,                     *** equidistant subintervalls. Returns a
  for i from 1 to n do             *** 2 dim array of coefficients: Element
    c[i,1] := fouriersin(f,i,nsub),*** [0,1] is the constant coefficient, [0,2]
    c[i,2] := fouriercos(f,i,nsub) *** is always 0, [i,1] contains the i-th sin
  endfor,                          *** and [i,2] the i-th cos coefficient
  c                                *** (i from 0 to n). 
endwith;

fouriersyn(c)::               *** Given the result of fouriercoef as argument,
with n::highindex(c,1) do     *** returns a function F such that F(t) is the
                              *** trigonometric polynom with coefficients 
                              *** computed by fouriercoef (i.e. F should be a
                              *** trigonometric approximation of the argument
                              *** f of fouriercoef).
  t ->          
    (c[0,1] + sum i->c[i,1]*sin(i*t) + c[i,2]*cos(i*t) from 1 to n endsum)/pi
endwith;

fourierpolys(f,l,factor) ::
if not listq(l) or nullq(l) then
  {}
else
  cons(fouriersyn(fouriercoef(f,first(l),first(l)*factor)),
                                              fourierpolys(f,rest(l),factor))
endif;
      
displayln("\n--- esl file \"fourier\" (Version ",fourier_version,", ",
                                     fourier_date,")");
