                  esl syntax overview        first version 27.01.2005
                  *******************             revision 29.01.2005
                                             last revision 06.02.2005           

Although esl semantics is scheme semantics, esl syntax is quit different.
The esl syntax is more standard.

esl syntax is converted to scheme syntax by esl and then evaluated as scheme
syntax.

Try
parseon();
This enables the output of the parsed esl input before evaluation. 
Note that esl symbols are prepended with a dollar sign. Do not enter this
dollar sign when working in esl.

parseoff();
turns off this feature.

If you want the display of the parsed esl input appear "pretty", use

prettyprinton();

in addition to parson();

In this case, the scheme (or guile?) procedure pretty-print is used to
display the result of parsing on with proper indentations (usually on several
lines). The pretty-print feature might not work on other scheme
implementations than guile.

An important feature of esl syntax, when compared to other standard syntax
languages: The comma (,) is not only used as delimiter in lists or similiar
situations (separating arguments in function), but als as delimiter between
statements (note that there is no real difference between expressions and
statements). The semicolon is not used for this latter purpose. The semicolon
is used to separate parsing units. Everything between two semicolons is
parsed as a unit. Therefore it must be a valid piece of the language.
Never use the semicolon within control constructs.

while [cond] do stat1; stat2 endwhile

will not work, because "while [cond] do stat1" is not a valid parse unit.
Consider the semicolon as an request to esl: "Parse and evaluate the preceding
input up to (and excluding) this semicolon".

New in esl version 0.06: In interactive esl, two consecutive newlines (hits
of the return key) can be recognized as "end of parse unit", i.e. as something
equivalent to a semicolon. This feature is off by default, it can be
switched on or off using
returnreturnon()
returnreturnoff()
respectively. It is ignored when reading input from files. Consider this
feature experimental.

Contrary to most scheme implementations, esl is case sensitive.
Note: Current guile (version 1.6.x) seems to be case sensitive by default.

Every variable must be explicitely defined (or "bound" within a control
construct). Defining a variable involves assigning a value.
Define variable using the :: operator (corresponding to scheme "define"
or variable binding in scheme control constructs like let):
 
var::[value]

The fundamental assignment operator is :=

var:=[value]

assigns [value] to var. var must have been "defined" before. := returns an
unspecified value. := is scheme's set!.
You can also use the simple colon
var:[value]
for assignment. : returns the assigned value. You can use something like
a:b:c:0
to assign 0 to the variables a, b, c. Use the colon only in interactive esl. 

1. With respect to syntax, there are functions, operators, control 
   constructs. See 'help "overview"'.
   There is special syntax for some important situations. See 11. below.
   The denotation "function" as used here refers to syntax, not semantics.
   What is called a function here may be a macro. For example
   push(ob,lis)
   is a macro invocation. It does not evaluate the second argument (lis) in
   standard way. Syntactically, push behaves like other functions.

2. Function calls have standard syntax: f(arg1, arg2, ...). There are
   other possibilities for zero and one argument functions. However, standard
   syntax (using parentheses) should always work.

3. Calls for functions without arguments must include the parentheses:
   parseon()
   or you must use the new syntax
   !parseon
   (this feature is included in version 0.05, patchlevel 15). Do not use
   this for the definition of zero argument functions. Also note that this
   does not work for macros. It is meant for procederes.

4. Functions with exactly one argument may be called without parentheses:

   sqrt 2          is equivalent to    sqrt(2)

   This does not apply when the function itself is the result of an
   evaluation.
   So
   x::-4, if x<0 then t->sqrt(-t) else sqrt endif (x);
   will work. The if ... endif construct evaluates to a procedure (depending
   on the sign of x, this procedure is either the usual square root function
   or the function that takes the square root of the negative of its argument.
   This function can be applied to x. However, if you remove the parentheses
   around x, the expression will become syntactically invalid.
   When in doubt, use parentheses.

5. There are prefix, postfix and infix operators. Operator precedence is
   controlled by "binding powers". See 'help "operators"'. When one-argument
   functions are used as prefix operators, they are a assigned a certain 
   "right binding power".

6. End-of-line-comments:   everything between *** and End of line (including
   ***) is considered a comment.

7. C-style comments:
   /* [anything but "/*", "*/"] */
   is considered as whitespace. It may contain linebreaks.
   C-style comments may be nested. Something like
   /* ...  /* ... */ ../* ... /* ... */ ...*/ ... */
   is a comment, when /* ... */ only occur in matching pairs.

8. String constants are enclosed in "  ". The usual escape sequences
   \\ \'  \"  \n will work. Note: \n might possibly not work with all scheme
   dialects. It is not part of the standard. Note also, that \n within
   strings, even if it is interpreted as newline character by the scheme
   implementation will possibly not be honoured by the write procedure. This
   latter is true for guile: \n inserts a newline in a string, but when output
   with write, a "hard" newline will be written.

9. There is a data type "symbol" (see scheme manual).

10. The single apostroph is a prefix operator. It is not used pairwise.
   animal :: 'dog
   means: A variable animal is defined its value is dog (the symbol dog).
   This is different from
   animal :: dog
   The latter will only work if dog has a value (is "defined"). If this is
   the case, the value of animal will (usually) not become dog, but the value
   of the variable dog.
   See also "quoted lists" below.

11. Special syntax
   There is special syntax (not covered by functions, operators and
   control constructs) for procedere definition, lists, quoted lists, arrays.

   Procedure definitions
   ---------------------
   Procedures may be defined with standard syntax using the -> operator
   (corresponding to scheme lambda):

   t->t*t+t
   or
   (x,y)->sin x * cos y
   You can define a variable with a procedere value using :: combined with
   ->.
   f::t->t*t+t
   
   Alternatively, the special syntax 
   f(t)::t*t+t
   for procedere definition can be used.
   
   Lists
   -----
   {el1, el2, ...}
   is equivalent to
   list(el1,el2, ...)
   first(op), rest(op) can be located left to an assignment sign, if op
   evaluates to a list. In this case, it acts like the scheme syntax
   set-car!, set-cdr!, respectively.

   Quoted lists
   ------------
   '(el1, el2, el3,  ...)
   is a list formed of the unevaluated Elements el1, el2, ...
   The elements may themselves be lists (enclosed in parentheses)). Parsing
   within '(...) still works:
   '(a,5+b)  ==> {a,{+,5,b}}
   The syntax '{el1, el2, ...} is valid, but usually not useful. It creates
   a list with the symbol list as first element.
   '{a,b} ==> {list, a, b}
   Note: The denotation ==> should be read as "evaluates to".
   An evaluation result of the form {a, {+, 5, b}} is not exactly the form
   used by esl for the display of result; esl results are still shown in
   scheme syntax.
   
   Arrays
   ------
   a::[l1..u1,l2..u2, ..., ln..un]
   creates an array with n dimensions. Index bounds may be any integers,
   the only condition is that li<=ui always holds.
   array element reference by a[i1,i2,...,in]. This may be on the left
   side of an assignment:
   a[i1,i2, ..., in] := [expression]
   An array element can hold any value.
