misc_num_version::"0.01", misc_num_date::"14.07.1999";

/* Roots of equations */

newtonlist(f,df,x,eps,nmax)::    *** newton method for searching roots
with                             *** of functions. Returns a list,
      n::0,                      *** containing one root and other
      error::0                   *** information. 
                                 *** f is the function, df is the
                                 *** derivative, eps is the allowed
                                 *** error (max of abs(f(x)), where
                                 *** x is the approximative root), 
do                               *** nmax is the maximal iteration count.
      while error=0 and abs(f(x))>eps and ++n<=nmax do
        if abs(df(x))<eps then
          error:=1
        else  
          x:=x-f(x)/df(x)
        endif
      endwhile,
      list(if error=1 then
           "df too small"
         elseif n>nmax then
           "aborted after n=nmax"
         else
          ""
        endif,
        "xn=",x,
        "f(xn)=",f(x),    
        "n=",min(n,nmax)
      )
endwith;

newton(f,df,x,eps,nmax)::       *** newton method, returns only one root
with                            *** see newtonlist 
   nt::newtonlist(f,df,x,eps,nmax)
do
   if first(nt)="df too small" then
      display("newton: abs(df) too small, iteration aborted.\n"),
      0
   elseif first(nt)="aborted after n=nmax" then
     display("newton: aborted after n=nmax iterations.\n"),
     0
   else
     third(nt)
   endif
endwith;

/* Example of an inverse function */

dtan(x)::1/cos(x)^2;               *** This is the derivative of tan

patan(x)::                         *** Newton's method to find inverse of tan
                                   *** (i.e. atan) for 0 <= x <= 99
if x > 21 then 
  newton(t->tan(t)-x,dtan,1.57075,1e-12,5000)
elseif x > 9 then
  newton(t->tan(t)-x,dtan,1.57075,1e-13,2000)
elseif x > 3 then
  newton(t->tan(t)-x,dtan,1.57075,1e-14,2000)
else
  newton(t->tan(t)-x,dtan,1.57075,1e-15,2000)
endif;
   

matan(x)::                       *** "Implementation" of atan for
if abs(x)>99 then                *** -99 <= x <= 99, with range check,
  "matan: argument out of range" *** seems to differ from built in atan
else                             *** by at most 2.2E-16 (the "machine
  if x<0 then                    *** epsilon").
    -patan(-x) 
  else 
    patan(x) 
  endif
endif;

/* Normal distribution */

nv(x)::(1/sqrt(2*pi))*exp(-x*x/2); *** Density of standard normal distribution

NVS(x)::       *** Distribution function of standard normal distribution
if x<-10 then 
  0
elseif x>10 then
  1
else 
  simpson(nv,-10,x,500)
endif;

NV(x,my,sigma)::NVS((x-my)/sigma); *** Distribution function for N(my,sigma)

displayln("\n--- esl file \"misc_num\" (Version ",misc_num_version,", ",
                                     misc_num_date,")");
